package fi.vtt.componentbee.behaviourparser_02_10_2008_14_29_56;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import fi.vtt.componentbee.behaviourlog.BehaviourPattern;
import fi.vtt.componentbee.behaviourlog.BehaviourPatternVisitor;
import fi.vtt.componentbee.behaviourlog.ParallelBehaviour;

public abstract class BehaviourPatternImpl implements BehaviourPattern, Node
{
	public ParallelBehaviour parallelBehaviour;
	protected final Hashtable<String, Object> attributes=new Hashtable<String, Object>();
	
	public BehaviourPatternImpl()
	{
		setIndex(NOT_DEFINED_VALUE);
		setType(NOT_DEFINED_VALUE);
		setAsynchronous(false);
	}

	protected static final boolean isCorrectType(
			final BehaviourPattern behaviourPattern, 
			final int types[]){
			
		if(types==null) return true;
		for (int i = 0; i < types.length; i++) {
			if(behaviourPattern.getType()==types[i]) return true;
		}
		return false;
	}
	
	public final void visit(
			final BehaviourPatternVisitor behaviourPatternVisitor,
			final int types[],
			final boolean deep){
			
		// The visitor is called only if the pattern type is correct.
		if(isCorrectType(this,types)) behaviourPatternVisitor.accept(this);

		if(deep && hasChilds())
		{
			int numberOfChilds=jjtGetNumChildren();
			BehaviourPattern childBehaviourPattern;
			for (int i = 0; i < numberOfChilds; i++) {
				childBehaviourPattern=(BehaviourPattern) jjtGetChild(i);
				childBehaviourPattern.visit(behaviourPatternVisitor,types, deep);
			}
		}
	}

	/**
	 * If deep has value true, the behaviour pattern and its child behavior patterns are visited.
	 * @param behaviourPatternVisitor
	 * @param deep
	 */
	public final void visit(
			final BehaviourPatternVisitor behaviourPatternVisitor,
			final boolean deep){
			
		visit(behaviourPatternVisitor, null, deep);
	}


	public final boolean hasChilds(){
		return jjtGetNumChildren()>0;
	}

	public final Enumeration getChildPatterns(){
		/*
		Vector vector=new Vector();
		int numberOfChilds=jjtGetNumChildren();
		for (int i = 0; i < numberOfChilds; i++) {
			vector.add(jjtGetChild(i));
		}
		return vector.elements();
		*/
		
		return new BehaviourPatternEnumeration(this);
	}

	public final BehaviourPattern getFirst(int type){
		int numberOfChilds=jjtGetNumChildren();
		BehaviourPattern childBehaviourPattern;
		for (int i = 0; i < numberOfChilds; i++) {
			childBehaviourPattern=(BehaviourPattern) jjtGetChild(i);

			// Child is returned if its type is correct.
			if(type==childBehaviourPattern.getType()){
				return childBehaviourPattern;
			}
			else{
				childBehaviourPattern=childBehaviourPattern.getFirst(type);
				if(childBehaviourPattern!=null) return childBehaviourPattern;
			}
		}
		return null;
	}

	public final BehaviourPattern getLast(int type){
		int numberOfChilds=jjtGetNumChildren();
		BehaviourPattern childBehaviourPattern;
		BehaviourPattern lastChildBehaviourPattern;
		for (int i = 0; i < numberOfChilds; i++) {
			childBehaviourPattern=(BehaviourPattern) jjtGetChild(numberOfChilds-i-1);

			// Last child is found first.
			lastChildBehaviourPattern=childBehaviourPattern.getLast(type);
			if(lastChildBehaviourPattern!=null) return lastChildBehaviourPattern;

			// This child is returned if its type is correct.
			else if(type==childBehaviourPattern.getType()){
				return childBehaviourPattern;
			}
		}
		return null;
	}

	public String getTextualPresentation(final String indent,final boolean deep){
		StringBuffer stringBuffer=new StringBuffer(10000);
		getTextualPresentation(stringBuffer,indent,deep);
		return stringBuffer.toString();
	}

	public void getTextualPresentation(
			final StringBuffer stringBuffer,
			final String indent,
			final boolean deep){
		
		if(getName()!=null) stringBuffer.append("\n"+indent+getName());
		if(isAsynchronous()) stringBuffer.append("(ASYNC)");
		if(getIndex()>0) stringBuffer.append("["+getIndex()+"]");

		String attributeName;
		Enumeration<String> attributeNames=getAttributeNames();
		while (attributeNames.hasMoreElements()) {
			attributeName = (String) attributeNames.nextElement();
			
			if(attributeName.equals(ATTRIBUTE_NAME)) continue;
			else if(attributeName.equals(ATTRIBUTE_ISASYNCRHONOUS)) continue;
			else if(attributeName.equals(ATTRIBUTE_INDEX)) continue;
			
			stringBuffer.append("\n"+indent+"  "+attributeName+"="+getAttributeValue(attributeName));
		}
	
		if(hasChilds()&&deep){
			stringBuffer.append('{');
			
			BehaviourPattern childElement;
			final int numberOfChilds=jjtGetNumChildren();
			for (int i = 0; i < numberOfChilds; i++) {
				childElement=(BehaviourPattern) jjtGetChild(i);
				childElement.getTextualPresentation(stringBuffer,indent+"   ",deep);
			}
			
			stringBuffer.append("\n"+indent+"}");
		}
	}

	public void interpret()
	{
		throw new UnsupportedOperationException(); // It better not come here.
	}


 	public void setAttribute(String name, Object value){
 		attributes.put(name, value);
 	}

	public Object getAttributeValue(String name){
		return attributes.get(name);
 	}

	public Enumeration getAttributeNames(){
		return attributes.keys();
 	}

	public boolean isAttributeDefined(String name){
		return attributes.containsKey(name);
 	}
	
	public void setName(String name){
		setAttribute(ATTRIBUTE_NAME, name);
	}
	public void setIndex(long index){
		setAttribute(ATTRIBUTE_INDEX, new Long(index));	
	}
	public void setType(int type){
		setAttribute(ATTRIBUTE_TYPE, new Integer(type));	
	}
	public void setUseCaseDescription(String useCaseDescription){
		setAttribute(ATTRIBUTE_USECASEDESCRIPTION, useCaseDescription);	
	}
	
	public void setAsynchronous(boolean isAsynchronous){
		setAttribute(ATTRIBUTE_ISASYNCRHONOUS, new Boolean(isAsynchronous));	
	}
	
	public String getName(){
		return (String) getAttributeValue(ATTRIBUTE_NAME);	
	}

	public long getIndex(){
		return ((Long) getAttributeValue(ATTRIBUTE_INDEX)).longValue();
	}

	public int getType(){
		return ((Integer) getAttributeValue(ATTRIBUTE_TYPE)).intValue();
	}

	public boolean isAsynchronous(){
		return ((Boolean) getAttributeValue(ATTRIBUTE_ISASYNCRHONOUS)).booleanValue();
	}

	public String getUseCaseDescription(){
		return (String) getAttributeValue(ATTRIBUTE_USECASEDESCRIPTION);
	}

	public long getBeginTime(){
		return ((Long) getAttributeValue(ATTRIBUTE_BEGINTIME)).longValue();
	}
	
	public long getEndTime(){
		return ((Long) getAttributeValue(ATTRIBUTE_ENDTIME)).longValue();
	}
	
	public long getDuration(){
		return ((Long) getAttributeValue(ATTRIBUTE_DURATION)).longValue();
	}

	public long getMaxMemoryUsage(){
		return ((Long) getAttributeValue(ATTRIBUTE_MAXMEMORYUSAGE)).longValue();
	}
	
	public long getMinMemoryUsage(){
		return ((Long) getAttributeValue(ATTRIBUTE_MINMEMORYUSAGE)).longValue();
	}
	
	public long getAverageMemoryUsage(){
		return ((Long) getAttributeValue(ATTRIBUTE_AVERAGEMEMORYUSAGE)).longValue();
	}

	public long getChangeInMemoryUsage(){
		return ((Long) getAttributeValue(ATTRIBUTE_CHANGEINMEMORYUSAGE)).longValue();
	}

	public long getMeanTimeToRepair(){
		return ((Long) getAttributeValue(ATTRIBUTE_MEANTIMETOREPAIR)).longValue();
	}
	
	public long getMeanTimeToFailure(){
		return ((Long) getAttributeValue(ATTRIBUTE_MEANTIMETOFAILURE)).longValue();
	}
	
	public double getProbabilityOfFailure(){
		return ((Double) getAttributeValue(ATTRIBUTE_PROBABILITYOFFAILURE)).doubleValue();
	}
}