package fi.vtt.smallclientserverexample.evaluators;

import fi.vtt.componentbee.behaviourlog.BehaviourPattern;
import fi.vtt.componentbee.behaviourlog.Evaluator;
import fi.vtt.componentbee.behaviourlog.Message;
import fi.vtt.componentbee.behaviourlog.ProcessingMonitor;
import fi.vtt.componentbee.behaviourlog.abstractevaluator.AbstractPatternEvaluator;
import fi.vtt.componentbee.behaviourlog.abstractevaluator.UseCaseEvaluationResult;

public class EvaluatorForSmallClientServerApplication extends AbstractPatternEvaluator implements Evaluator{
	/**
	 * Returns true if the input values that are provided in the message are correct.
	 * If not the message sender is set to be a failure participant in the use case.
	 * 
	 * @param message
	 * @param useCase
	 * @param messageBehaviourPattern
	 * @param useCaseEvaluationResults
	 * @param processingMonitor
	 * @return
	 */
	public boolean evaluateInputValues(
			final Message message,
			final BehaviourPattern useCase,
			final BehaviourPattern messageBehaviourPattern,
			final UseCaseEvaluationResult useCaseEvaluationResults,
			final ProcessingMonitor processingMonitor){

		if(message.getMessageName().equals("response"))
		{
			// The server has caused a failure if it does send a failure response.
			String responseValue=(String) message.getInputAttribute(0);
			boolean isFailureParticipant=responseValue.startsWith("failure_response");
			useCaseEvaluationResults.addParticipant("fi.vtt.smallclientserverexample.impl.ServerImpl",isFailureParticipant);
		}
		if(useCase.getName().equals("IngoredRequest"))
		{
			// The server is caused a failure if it does not deliver a response.
			useCaseEvaluationResults.addParticipant("fi.vtt.smallclientserverexample.impl.ServerImpl",true);
		}
		return true;
	}

	/**
	 * Returns true if the message's return value is correct.
	 * If not the message receiver is set to be a failure participant in the use case.
	 * 
	 * @param returnValue
	 * @param message
	 * @param useCase
	 * @param messageBehaviourPattern
	 * @param useCaseEvaluationResults
	 * @param processingMonitor
	 * @return
	 */
	public boolean evaluateReturnValue(
			final Message message,
			final BehaviourPattern useCase,
			final BehaviourPattern messageBehaviourPattern,
			final UseCaseEvaluationResult useCaseEvaluationResults,
			final ProcessingMonitor processingMonitor){return true;}

	/**
	 * Returns true if the message's thrown value is correct.
	 * If not the message receiver is set to be a failure participant in the use case.
	 * 
	 * @param thrownValue
	 * @param message
	 * @param useCase
	 * @param messageBehaviourPattern
	 * @param useCaseEvaluationResults
	 * @param processingMonitor
	 * @return
	 */
	public boolean evaluateThrownException(
			final Message message,
			final BehaviourPattern useCase,
			final BehaviourPattern messageBehaviourPattern,
			final UseCaseEvaluationResult useCaseEvaluationResults,
			final ProcessingMonitor processingMonitor){return true;}

}
